@@ -17,20 +17,27 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 
-from __future__ import print_function
+from __future__ import print_function, absolute_import
 
+import logging
 import os
-from optparse import OptionGroup
-from nss import nss
-from nss.error import NSPRError
+from optparse import OptionGroup  # pylint: disable=deprecated-module
 import gssapi
 
-from ipapython import admintool, certmonger, ipautil
+from ipalib.constants import RENEWAL_CA_NAME, RENEWAL_REUSE_CA_NAME, IPA_CA_CN
+from ipalib.install import certmonger, certstore
+from ipapython import admintool, ipautil
+from ipapython.certdb import (EMPTY_TRUST_FLAGS,
+                              EXTERNAL_CA_TRUST_FLAGS,
+                              TrustFlags,
+                              parse_trust_flags)
 from ipapython.dn import DN
 from ipaplatform.paths import paths
-from ipalib import api, errors, x509, certstore
+from ipalib import api, errors, x509
 from ipaserver.install import certs, cainstance, installutils
 
+logger = logging.getLogger(__name__)
+
 
 class CACertManage(admintool.AdminTool):
     command_name = 'ipa-cacert-manage'
@@ -58,6 +65,18 @@ class CACertManage(admintool.AdminTool):
             "--external-ca", dest='self_signed',
             action='store_false',
             help="Sign the renewed certificate by external CA")
+        ext_cas = tuple(x.value for x in cainstance.ExternalCAType)
+        renew_group.add_option(
+            "--external-ca-type", dest="external_ca_type",
+            type="choice", choices=ext_cas,
+            metavar="{{{0}}}".format(",".join(ext_cas)),
+            help="Type of the external CA. Default: generic")
+        renew_group.add_option(
+            "--external-ca-profile", dest="external_ca_profile",
+            type='constructor', constructor=cainstance.ExternalCAProfile,
+            default=None, metavar="PROFILE-SPEC",
+            help="Specify the certificate profile/template to use "
+                 "at the external CA")
         renew_group.add_option(
             "--external-cert-file", dest="external_cert_files",
             action="append", metavar="FILE",
@@ -85,7 +104,6 @@ class CACertManage(admintool.AdminTool):
             parser.error("command not provided")
 
         command = self.command = self.args[0]
-        options = self.options
 
         if command == 'renew':
             pass
@@ -97,25 +115,21 @@ class CACertManage(admintool.AdminTool):
 
     def run(self):
         command = self.command
-        options = self.options
 
-        api.bootstrap(in_server=True)
+        api.bootstrap(in_server=True, confdir=paths.ETC_IPA)
         api.finalize()
 
-        if ((command == 'renew' and options.external_cert_files) or
-            command == 'install'):
-            self.ldap_connect()
+        self.ldap_connect()
 
         try:
             if command == 'renew':
-                rc = self.renew()
+                return self.renew()
             elif command == 'install':
-                rc = self.install()
+                return self.install()
+            else:
+                raise NotImplementedError
         finally:
-            if api.Backend.ldap2.isconnected():
-                api.Backend.ldap2.disconnect()
-
-        return rc
+            api.Backend.ldap2.disconnect()
 
     def ldap_connect(self):
         password = self.options.password
@@ -133,28 +147,36 @@ class CACertManage(admintool.AdminTool):
                 raise admintool.ScriptError(
                     "Directory Manager password required")
 
-        api.Backend.ldap2.connect(bind_dn=DN(('cn', 'Directory Manager')), bind_pw=password)
-
-
-    def renew(self):
-        ca = cainstance.CAInstance(api.env.realm, certs.NSS_DIR)
-        if not ca.is_configured():
-            raise admintool.ScriptError("CA is not configured on this system")
+        api.Backend.ldap2.connect(bind_pw=password)
 
+    def _get_ca_request_id(self, ca_name):
+        """Lookup tracking request for IPA CA, using given ca-name."""
         criteria = {
             'cert-database': paths.PKI_TOMCAT_ALIAS_DIR,
             'cert-nickname': self.cert_nickname,
-            'ca-name': 'dogtag-ipa-ca-renew-agent',
+            'ca-name': ca_name,
         }
-        self.request_id = certmonger.get_request_id(criteria)
+        return certmonger.get_request_id(criteria)
+
+    def renew(self):
+        ca = cainstance.CAInstance(api.env.realm)
+        if not ca.is_configured():
+            raise admintool.ScriptError("CA is not configured on this system")
+
+        self.request_id = self._get_ca_request_id(RENEWAL_CA_NAME)
         if self.request_id is None:
-            raise admintool.ScriptError(
-                "CA certificate is not tracked by certmonger")
-        self.log.debug(
+            # if external CA renewal was interrupted, the request may have
+            # been left with the "dogtag-ipa-ca-renew-agent-reuse" CA;
+            # look for it too
+            self.request_id = self._get_ca_request_id(RENEWAL_REUSE_CA_NAME)
+            if self.request_id is None:
+                raise admintool.ScriptError(
+                    "CA certificate is not tracked by certmonger")
+        logger.debug(
             "Found certmonger request id %r", self.request_id)
 
         db = certs.CertDB(api.env.realm, nssdir=paths.PKI_TOMCAT_ALIAS_DIR)
-        cert = db.get_cert_from_db(self.cert_nickname, pem=False)
+        cert = db.get_cert_from_db(self.cert_nickname)
 
         options = self.options
         if options.external_cert_files:
@@ -163,7 +185,7 @@ class CACertManage(admintool.AdminTool):
         if options.self_signed is not None:
             self_signed = options.self_signed
         else:
-            self_signed = x509.is_self_signed(cert, x509.DER)
+            self_signed = cert.is_self_signed()
 
         if self_signed:
             return self.renew_self_signed(ca)
@@ -173,19 +195,52 @@ class CACertManage(admintool.AdminTool):
     def renew_self_signed(self, ca):
         print("Renewing CA certificate, please wait")
 
+        msg = "You cannot specify {} when renewing a self-signed CA"
+        if self.options.external_ca_type:
+            raise admintool.ScriptError(msg.format("--external-ca-type"))
+        if self.options.external_ca_profile:
+            raise admintool.ScriptError(msg.format("--external-ca-profile"))
+
         try:
             ca.set_renewal_master()
         except errors.NotFound:
             raise admintool.ScriptError("CA renewal master not found")
 
-        self.resubmit_request(ca, 'caCACert')
+        self.resubmit_request()
+
+        db = certs.CertDB(api.env.realm, nssdir=paths.PKI_TOMCAT_ALIAS_DIR)
+        cert = db.get_cert_from_db(self.cert_nickname)
+        update_ipa_ca_entry(api, cert)
 
         print("CA certificate successfully renewed")
 
     def renew_external_step_1(self, ca):
         print("Exporting CA certificate signing request, please wait")
 
-        self.resubmit_request(ca, 'ipaCSRExport')
+        options = self.options
+
+        if not options.external_ca_type:
+            options.external_ca_type = cainstance.ExternalCAType.GENERIC.value
+
+        if options.external_ca_type == cainstance.ExternalCAType.MS_CS.value \
+                and options.external_ca_profile is None:
+            options.external_ca_profile = cainstance.MSCSTemplateV1(u"SubCA")
+
+        if options.external_ca_profile is not None:
+            # check that profile is valid for the external ca type
+            if options.external_ca_type \
+                    not in options.external_ca_profile.valid_for:
+                raise admintool.ScriptError(
+                    "External CA profile specification '{}' "
+                    "cannot be used with external CA type '{}'."
+                    .format(
+                        options.external_ca_profile.unparsed_input,
+                        options.external_ca_type)
+                    )
+
+        self.resubmit_request(
+            RENEWAL_REUSE_CA_NAME,
+            profile=options.external_ca_profile)
 
         print(("The next step is to get %s signed by your CA and re-run "
               "ipa-cacert-manage as:" % paths.IPA_CA_CSR))
@@ -198,58 +253,47 @@ class CACertManage(admintool.AdminTool):
 
         options = self.options
         conn = api.Backend.ldap2
+
+        old_spki = old_cert.public_key_info_bytes
+
         cert_file, ca_file = installutils.load_external_cert(
-            options.external_cert_files, x509.subject_base())
+            options.external_cert_files, DN(old_cert.subject))
 
-        nss_cert = None
-        nss.nss_init(paths.PKI_TOMCAT_ALIAS_DIR)
-        try:
-            nss_cert = x509.load_certificate(old_cert, x509.DER)
-            subject = nss_cert.subject
-            der_subject = x509.get_der_subject(old_cert, x509.DER)
-            #pylint: disable=E1101
-            pkinfo = nss_cert.subject_public_key_info.format()
-            #pylint: enable=E1101
+        with open(cert_file.name, 'rb') as f:
+            new_cert_data = f.read()
+        new_cert = x509.load_pem_x509_certificate(new_cert_data)
+        new_spki = new_cert.public_key_info_bytes
 
-            nss_cert = x509.load_certificate_from_file(cert_file.name)
-            cert = nss_cert.der_data
-            if nss_cert.subject != subject:
-                raise admintool.ScriptError(
-                    "Subject name mismatch (visit "
-                    "http://www.freeipa.org/page/Troubleshooting for "
-                    "troubleshooting guide)")
-            if x509.get_der_subject(cert, x509.DER) != der_subject:
-                raise admintool.ScriptError(
-                    "Subject name encoding mismatch (visit "
-                    "http://www.freeipa.org/page/Troubleshooting for "
-                    "troubleshooting guide)")
-            #pylint: disable=E1101
-            if nss_cert.subject_public_key_info.format() != pkinfo:
-                raise admintool.ScriptError(
-                    "Subject public key info mismatch (visit "
-                    "http://www.freeipa.org/page/Troubleshooting for "
-                    "troubleshooting guide)")
-            #pylint: enable=E1101
-        finally:
-            del nss_cert
-            nss.nss_shutdown()
+        if new_cert.subject != old_cert.subject:
+            raise admintool.ScriptError(
+                "Subject name mismatch (visit "
+                "http://www.freeipa.org/page/Troubleshooting for "
+                "troubleshooting guide)")
+        if new_cert.subject_bytes != old_cert.subject_bytes:
+            raise admintool.ScriptError(
+                "Subject name encoding mismatch (visit "
+                "http://www.freeipa.org/page/Troubleshooting for "
+                "troubleshooting guide)")
+        if new_spki != old_spki:
+            raise admintool.ScriptError(
+                "Subject public key info mismatch (visit "
+                "http://www.freeipa.org/page/Troubleshooting for "
+                "troubleshooting guide)")
 
         with certs.NSSDatabase() as tmpdb:
-            pw = ipautil.write_tmp_file(ipautil.ipa_generate_password())
-            tmpdb.create_db(pw.name)
-            tmpdb.add_cert(old_cert, 'IPA CA', 'C,,')
+            tmpdb.create_db()
+            tmpdb.add_cert(old_cert, 'IPA CA', EXTERNAL_CA_TRUST_FLAGS)
 
             try:
-                tmpdb.add_cert(cert, 'IPA CA', 'C,,')
+                tmpdb.add_cert(new_cert, 'IPA CA', EXTERNAL_CA_TRUST_FLAGS)
             except ipautil.CalledProcessError as e:
                 raise admintool.ScriptError(
                     "Not compatible with the current CA certificate: %s" % e)
 
             ca_certs = x509.load_certificate_list_from_file(ca_file.name)
             for ca_cert in ca_certs:
-                tmpdb.add_cert(ca_cert.der_data, str(ca_cert.subject), 'C,,')
-            del ca_certs
-            del ca_cert
+                tmpdb.add_cert(
+                    ca_cert, str(DN(ca_cert.subject)), EXTERNAL_CA_TRUST_FLAGS)
 
             try:
                 tmpdb.verify_ca_cert_validity('IPA CA')
@@ -266,38 +310,54 @@ class CACertManage(admintool.AdminTool):
                 except RuntimeError:
                     break
                 certstore.put_ca_cert_nss(
-                    conn, api.env.basedn, ca_cert, nickname, ',,')
+                    conn,
+                    api.env.basedn,
+                    ca_cert,
+                    nickname,
+                    EMPTY_TRUST_FLAGS)
 
         dn = DN(('cn', self.cert_nickname), ('cn', 'ca_renewal'),
                 ('cn', 'ipa'), ('cn', 'etc'), api.env.basedn)
+
         try:
             entry = conn.get_entry(dn, ['usercertificate'])
-            entry['usercertificate'] = [cert]
+            entry['usercertificate'] = [new_cert]
             conn.update_entry(entry)
         except errors.NotFound:
             entry = conn.make_entry(
                 dn,
                 objectclass=['top', 'pkiuser', 'nscontainer'],
                 cn=[self.cert_nickname],
-                usercertificate=[cert])
+                usercertificate=[new_cert])
             conn.add_entry(entry)
         except errors.EmptyModlist:
             pass
 
+        update_ipa_ca_entry(api, new_cert)
+
         try:
             ca.set_renewal_master()
         except errors.NotFound:
             raise admintool.ScriptError("CA renewal master not found")
 
-        self.resubmit_request(ca, 'ipaRetrieval')
+        self.resubmit_request(RENEWAL_REUSE_CA_NAME)
 
         print("CA certificate successfully renewed")
 
-    def resubmit_request(self, ca, profile):
+    def resubmit_request(self, ca=RENEWAL_CA_NAME, profile=None):
         timeout = api.env.startup_timeout + 60
 
-        self.log.debug("resubmitting certmonger request '%s'", self.request_id)
-        certmonger.resubmit_request(self.request_id, profile=profile)
+        cm_profile = None
+        if isinstance(profile, cainstance.MSCSTemplateV1):
+            cm_profile = profile.unparsed_input
+
+        cm_template = None
+        if isinstance(profile, cainstance.MSCSTemplateV2):
+            cm_template = profile.unparsed_input
+
+        logger.debug("resubmitting certmonger request '%s'", self.request_id)
+        certmonger.resubmit_request(self.request_id, ca=ca, profile=cm_profile,
+                                    template_v2=cm_template, is_ca=True)
         try:
             state = certmonger.wait_for_request(self.request_id, timeout)
         except RuntimeError:
@@ -310,8 +370,10 @@ class CACertManage(admintool.AdminTool):
                 "Error resubmitting certmonger request '%s', "
                 "please check the request manually" % self.request_id)
 
-        self.log.debug("modifying certmonger request '%s'", self.request_id)
-        certmonger.modify(self.request_id, profile='ipaCACertRenewal')
+        logger.debug("modifying certmonger request '%s'", self.request_id)
+        certmonger.modify(self.request_id,
+                          ca=RENEWAL_CA_NAME,
+                          profile='', template_v2='')
 
     def install(self):
         print("Installing CA certificate, please wait")
@@ -319,21 +381,15 @@ class CACertManage(admintool.AdminTool):
         options = self.options
         cert_filename = self.args[1]
 
-        nss_cert = None
         try:
-            try:
-                nss_cert = x509.load_certificate_from_file(cert_filename)
-            except IOError as e:
-                raise admintool.ScriptError(
-                    "Can't open \"%s\": %s" % (cert_filename, e))
-            except (TypeError, NSPRError, ValueError) as e:
-                raise admintool.ScriptError("Not a valid certificate: %s" % e)
-            subject = nss_cert.subject
-            cert = nss_cert.der_data
-        finally:
-            del nss_cert
+            cert = x509.load_certificate_from_file(cert_filename)
+        except IOError as e:
+            raise admintool.ScriptError(
+                "Can't open \"%s\": %s" % (cert_filename, e))
+        except (TypeError, ValueError) as e:
+            raise admintool.ScriptError("Not a valid certificate: %s" % e)
 
-        nickname = options.nickname or str(subject)
+        nickname = options.nickname or str(DN(cert.subject))
 
         ca_certs = certstore.get_ca_certs_nss(api.Backend.ldap2,
                                               api.env.basedn,
@@ -341,9 +397,8 @@ class CACertManage(admintool.AdminTool):
                                               False)
 
         with certs.NSSDatabase() as tmpdb:
-            pw = ipautil.write_tmp_file(ipautil.ipa_generate_password())
-            tmpdb.create_db(pw.name)
-            tmpdb.add_cert(cert, nickname, 'C,,')
+            tmpdb.create_db()
+            tmpdb.add_cert(cert, nickname, EXTERNAL_CA_TRUST_FLAGS)
             for ca_cert, ca_nickname, ca_trust_flags in ca_certs:
                 tmpdb.add_cert(ca_cert, ca_nickname, ca_trust_flags)
 
@@ -355,11 +410,25 @@ class CACertManage(admintool.AdminTool):
                     "http://www.freeipa.org/page/Troubleshooting for "
                     "troubleshooting guide)" % e)
 
-        trust_flags = options.trust_flags
-        if ((set(trust_flags) - set(',CPTcgpuw')) or
-            len(trust_flags.split(',')) != 3):
+        trust_flags = options.trust_flags.split(',')
+        if (set(options.trust_flags) - set(',CPTcgpuw') or
+                len(trust_flags) not in [3, 4]):
             raise admintool.ScriptError("Invalid trust flags")
 
+        extra_flags = trust_flags[3:]
+        extra_usages = set()
+        if extra_flags:
+            if 'C' in extra_flags[0]:
+                extra_usages.add(x509.EKU_PKINIT_KDC)
+            if 'T' in extra_flags[0]:
+                extra_usages.add(x509.EKU_PKINIT_CLIENT_AUTH)
+
+        trust_flags = parse_trust_flags(','.join(trust_flags[:3]))
+        trust_flags = TrustFlags(trust_flags.has_key,
+                                 trust_flags.trusted,
+                                 trust_flags.ca,
+                                 trust_flags.usages | extra_usages)
+
         try:
             certstore.put_ca_cert_nss(
                 api.Backend.ldap2, api.env.basedn, cert, nickname, trust_flags)
@@ -368,3 +437,21 @@ class CACertManage(admintool.AdminTool):
                 "Failed to install the certificate: %s" % e)
 
         print("CA certificate successfully installed")
+
+
+def update_ipa_ca_entry(api, cert):
+    """
+    The Issuer DN of the IPA CA may have changed.  Update the IPA CA entry.
+
+    :param api: finalised API object, with *connected* LDAP backend
+    :param cert: a python-cryptography Certificate object
+
+    """
+    try:
+        entry = api.Backend.ldap2.get_entry(
+            DN(('cn', IPA_CA_CN), api.env.container_ca, api.env.basedn),
+            ['ipacaissuerdn'])
+        entry['ipacaissuerdn'] = [DN(cert.issuer)]
+        api.Backend.ldap2.update_entry(entry)
+    except errors.EmptyModlist:
+        pass
