@@ -1,12 +1,16 @@
-import shlex
-import sys
 import contextlib
+import os
+import shlex
+import subprocess
+import sys
+import tempfile
+import unittest
 
-import nose
 import six
 from six import StringIO
 
 from ipatests import util
+from ipatests.test_ipalib.test_x509 import goodcert_headers
 from ipalib import api, errors
 import pytest
 
@@ -15,8 +19,12 @@ if six.PY3:
 
 TEST_ZONE = u'zoneadd.%(domain)s' % api.env
 
+HERE = os.path.abspath(os.path.dirname(__file__))
+BASE_DIR = os.path.abspath(os.path.join(HERE, os.pardir, os.pardir))
+
 
 @pytest.mark.tier0
+@pytest.mark.needs_ipaapi
 class TestCLIParsing(object):
     """Tests that commandlines are correctly parsed to Command keyword args
     """
@@ -37,7 +45,7 @@ class TestCLIParsing(object):
         try:
             api.Command[command_name](**kw)
         except errors.NetworkError:
-            raise nose.SkipTest('%r: Server not available: %r' %
+            raise unittest.SkipTest('%r: Server not available: %r' %
                                 (self.__module__, api.env.xmlrpc_uri))
 
     @contextlib.contextmanager
@@ -51,6 +59,9 @@ class TestCLIParsing(object):
     def test_ping(self):
         self.check_command('ping', 'ping')
 
+    def test_plugins(self):
+        self.check_command('plugins', 'plugins')
+
     def test_user_show(self):
         self.check_command('user-show admin', 'user_show', uid=u'admin')
 
@@ -113,7 +124,7 @@ class TestCLIParsing(object):
         try:
             self.run_command('dnszone_add', idnsname=TEST_ZONE)
         except errors.NotFound:
-            raise nose.SkipTest('DNS is not configured')
+            raise unittest.SkipTest('DNS is not configured')
         try:
             self.run_command('dnsrecord_add',
                 dnszoneidnsname=TEST_ZONE,
@@ -141,7 +152,7 @@ class TestCLIParsing(object):
         try:
             self.run_command('dnszone_add', idnsname=TEST_ZONE)
         except errors.NotFound:
-            raise nose.SkipTest('DNS is not configured')
+            raise unittest.SkipTest('DNS is not configured')
         try:
             records = (u'1 1 E3B72BA346B90570EED94BE9334E34AA795CED23',
                        u'2 1 FD2693C1EFFC11A8D2BE57229212A04B45663791')
@@ -221,7 +232,7 @@ class TestCLIParsing(object):
             self.run_command(
                 'dnszone_add', idnsname=TEST_ZONE)
         except errors.NotFound:
-            raise nose.SkipTest('DNS is not configured')
+            raise unittest.SkipTest('DNS is not configured')
         try:
             self.run_command(
                 'dnsrecord_add',
@@ -302,3 +313,88 @@ class TestCLIParsing(object):
 
         if not adtrust_is_enabled:
             mockldap.del_entry(adtrust_dn)
+
+    def test_certfind(self):
+        with tempfile.NamedTemporaryFile() as f:
+            f.write(goodcert_headers)
+            f.flush()
+            self.check_command(
+                'cert_find --file={}'.format(f.name),
+                'cert_find',
+                file=goodcert_headers
+            )
+
+
+def test_cli_fsencoding():
+    # https://pagure.io/freeipa/issue/5887
+    env = {
+        key: value for key, value in os.environ.items()
+        if not key.startswith(('LC_', 'LANG'))
+    }
+    env['LC_ALL'] = 'C'
+    env['PYTHONPATH'] = BASE_DIR
+    p = subprocess.Popen(
+        [sys.executable, '-m', 'ipaclient', 'help'],
+        stdout=subprocess.PIPE,
+        stderr=subprocess.PIPE,
+        env=env,
+    )
+    out, err = p.communicate()
+    assert p.returncode > 0, (out, err)
+    assert b'System encoding must be UTF-8' in err, (out, err)
+
+
+IPA_NOT_CONFIGURED = b'IPA is not configured on this system'
+IPA_CLIENT_NOT_CONFIGURED = b'IPA client is not configured on this system'
+
+
+@pytest.mark.needs_ipaapi
+@pytest.mark.skipif(
+    os.geteuid() != 0 or os.path.isfile('/etc/ipa/default.conf'),
+    reason="Must have root privileges to run this test "
+           "and IPA must not be installed")
+@pytest.mark.parametrize(
+    "args, retcode, output, error",
+    [
+        # Commands delivered by the client pkg
+        (['ipa'], 1, None, IPA_CLIENT_NOT_CONFIGURED),
+        (['ipa-certupdate'], 1, None, IPA_CLIENT_NOT_CONFIGURED),
+        (['ipa-client-automount'], 1, IPA_CLIENT_NOT_CONFIGURED, None),
+        # Commands delivered by the server pkg
+        (['ipa-adtrust-install'], 2, None, IPA_NOT_CONFIGURED),
+        (['ipa-advise'], 2, None, IPA_NOT_CONFIGURED),
+        (['ipa-backup'], 2, None, IPA_NOT_CONFIGURED),
+        (['ipa-cacert-manage'], 2, None, IPA_NOT_CONFIGURED),
+        (['ipa-ca-install'], 1, None,
+         b'IPA server is not configured on this system'),
+        (['ipa-compat-manage'], 2, None, IPA_NOT_CONFIGURED),
+        (['ipa-csreplica-manage'], 1, None, IPA_NOT_CONFIGURED),
+        (['ipactl', 'status'], 4, None, b'IPA is not configured'),
+        (['ipa-dns-install'], 2, None, IPA_NOT_CONFIGURED),
+        (['ipa-kra-install'], 2, None, IPA_NOT_CONFIGURED),
+        (['ipa-ldap-updater',
+          '/usr/share/ipa/updates/05-pre_upgrade_plugins.update'],
+         2, None, IPA_NOT_CONFIGURED),
+        (['ipa-managed-entries'], 2, None, IPA_NOT_CONFIGURED),
+        (['ipa-nis-manage'], 2, None, IPA_NOT_CONFIGURED),
+        (['ipa-pkinit-manage'], 2, None, IPA_NOT_CONFIGURED),
+        (['ipa-replica-manage', 'list'], 1, IPA_NOT_CONFIGURED, None),
+        (['ipa-server-certinstall'], 2, None, IPA_NOT_CONFIGURED),
+        (['ipa-server-upgrade'], 2, None, IPA_NOT_CONFIGURED),
+        (['ipa-winsync-migrate'], 1, None, IPA_NOT_CONFIGURED)
+    ])
+def test_command_ipa_not_installed(args, retcode, output, error):
+    """
+    Test that the commands properly return that IPA client|server is not
+    configured on this system.
+    Launch the command specified in args.
+    Check that the exit code is as expected and that stdout and stderr
+    contain the expected strings.
+    """
+    p = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+    out, err = p.communicate()
+    assert retcode == p.returncode
+    if output:
+        assert output in out
+    if error:
+        assert error in err
