@@ -7,24 +7,37 @@ import re
 import pytest
 
 from ipatests.test_integration.base import IntegrationTest
-from ipatests.test_integration import tasks
-from ipatests.test_integration.env_config import get_global_config
+from ipatests.pytest_ipa.integration import tasks
+from ipatests.pytest_ipa.integration.env_config import get_global_config
 from ipalib.constants import DOMAIN_SUFFIX_NAME
 from ipatests.util import assert_deepequal
 
 config = get_global_config()
 reasoning = "Topology plugin disabled due to domain level 0"
 
+
+def find_segment(master, replica):
+    result = master.run_command(['ipa', 'topologysegment-find',
+                                 DOMAIN_SUFFIX_NAME]).stdout_text
+    segment_re = re.compile('Left node: (?P<left>\S+)\n.*Right node: '
+                            '(?P<right>\S+)\n')
+    allsegments = segment_re.findall(result)
+    for segment in allsegments:
+        if master.hostname in segment and replica.hostname in segment:
+            return '-to-'.join(segment)
+
+
 @pytest.mark.skipif(config.domain_level == 0, reason=reasoning)
 class TestTopologyOptions(IntegrationTest):
     num_replicas = 2
     topology = 'star'
-    rawsegment_re = ('Segment name: (?P<name>.*?)',
-                     '\s+Left node: (?P<lnode>.*?)',
-                     '\s+Right node: (?P<rnode>.*?)',
-                     '\s+Connectivity: (?P<connectivity>\S+)')
+    rawsegment_re = (r'Segment name: (?P<name>.*?)',
+                     r'\s+Left node: (?P<lnode>.*?)',
+                     r'\s+Right node: (?P<rnode>.*?)',
+                     r'\s+Connectivity: (?P<connectivity>\S+)')
     segment_re = re.compile("\n".join(rawsegment_re))
-    noentries_re = re.compile("Number of entries returned (\d+)")
+    noentries_re = re.compile(r"Number of entries returned (\d+)")
+    segmentnames_re = re.compile(r'.*Segment name: (\S+?)\n.*')
 
     @classmethod
     def install(cls, mh):
@@ -51,6 +64,7 @@ class TestTopologyOptions(IntegrationTest):
                               )
         return result
 
+
     def test_topology_updated_on_replica_install_remove(self):
         """
         Install and remove a replica and make sure topology information is
@@ -62,15 +76,11 @@ class TestTopologyOptions(IntegrationTest):
         tasks.kinit_admin(self.master)
         result1 = self.master.run_command(['ipa', 'topologysegment-find',
                                            DOMAIN_SUFFIX_NAME]).stdout_text
-        first_segment_name = "%s-to-%s" % (self.master.hostname,
-                                           self.replicas[0].hostname)
-        expected_segment = {
-            'connectivity': 'both',
-            'leftnode': self.master.hostname,
-            'name': first_segment_name,
-            'rightnode': self.replicas[0].hostname}
-        firstsegment = self.tokenize_topologies(result1)[0]
-        assert_deepequal(expected_segment, firstsegment)
+        segment_name = self.segmentnames_re.findall(result1)[0]
+        assert(self.master.hostname in segment_name), (
+            "Segment %s does not contain master hostname" % segment_name)
+        assert(self.replicas[0].hostname in segment_name), (
+            "Segment %s does not contain replica hostname" % segment_name)
         tasks.install_replica(self.master, self.replicas[1], setup_ca=False,
                               setup_dns=False)
         # We need to make sure topology information is consistent across all
@@ -87,8 +97,11 @@ class TestTopologyOptions(IntegrationTest):
         assert_deepequal(result3.stdout_text,  result4.stdout_text)
         # Now let's check that uninstalling the replica will update the topology
         # info on the rest of replicas.
-        tasks.uninstall_master(self.replicas[1])
+        # first step of uninstallation is removal of the replica on other
+        # master, then it can be uninstalled. Doing it the other way is also
+        # possible, but not reliable - some data might not be replicated.
         tasks.clean_replication_agreement(self.master, self.replicas[1])
+        tasks.uninstall_master(self.replicas[1])
         result5 = self.master.run_command(['ipa', 'topologysegment-find',
                                            DOMAIN_SUFFIX_NAME])
         num_entries = self.noentries_re.search(result5.stdout_text).group(1)
@@ -117,8 +130,10 @@ class TestTopologyOptions(IntegrationTest):
             "%s: segment not found" % segment['name'])
         # Remove master <-> replica2 segment and make sure that the changes get
         # there through replica1
-        deleteme = "%s-to-%s" % (self.master.hostname,
-                                 self.replicas[1].hostname)
+        # Since segment name can be one of master-to-replica2 or
+        # replica2-to-master, we need to determine the segment name dynamically
+
+        deleteme = find_segment(self.master, self.replicas[1])
         returncode, error = tasks.destroy_segment(self.master, deleteme)
         assert returncode == 0, error
         # Wait till replication ends and make sure replica1 does not have
@@ -153,9 +168,8 @@ class TestTopologyOptions(IntegrationTest):
         returncode, error = tasks.destroy_segment(self.master, "%s-to-%s" % replicas)
         assert returncode != 0, error1
         assert error.count(text) == 1, error2 % error
-        newseg, err = tasks.create_segment(self.master,
-                                           self.master,
-                                           self.replicas[1])
+        _newseg, err = tasks.create_segment(
+            self.master, self.master, self.replicas[1])
         assert err == "", err
         returncode, error = tasks.destroy_segment(self.master, "%s-to-%s" % replicas)
         assert returncode == 0, error
@@ -182,7 +196,7 @@ class TestCASpecificRUVs(IntegrationTest):
                "Certificate Server Replica"
                " Update Vectors" in res1.stdout_text), (
                "CA-specific RUVs are not displayed")
-        ruvid_re = re.compile(".*%s:389: (\d+).*" % replica.hostname)
+        ruvid_re = re.compile(r".*%s:389: (\d+).*" % replica.hostname)
         replica_ruvs = ruvid_re.findall(res1.stdout_text)
         # Find out the number of RUVids
         assert(len(replica_ruvs) == 2), (
@@ -225,80 +239,11 @@ class TestCASpecificRUVs(IntegrationTest):
         assert(res1.count(replica.hostname) == 2), (
             "Did not find proper number of replica hostname (%s) occurrencies"
             " in the command output: %s" % (replica.hostname, res1))
+
+        master.run_command(['ipa-replica-manage', 'del', replica.hostname,
+                            '-p', master.config.dirman_password])
         tasks.uninstall_master(replica)
         res2 = master.run_command(['ipa-replica-manage', 'list-ruv', '-p',
                                   master.config.dirman_password]).stdout_text
         assert(replica.hostname not in res2), (
             "Replica RUVs were not clean during replica uninstallation")
-
-
-class TestReplicaManageDel(IntegrationTest):
-    domain_level = 0
-    topology = 'star'
-    num_replicas = 3
-
-    def test_replica_managed_del_domlevel0(self):
-        """
-        http://www.freeipa.org/page/V4/Manage_replication_topology_4_4/
-        Test_Plan#Test_case:_ipa-replica-manage_del_with_turned_off_replica
-        _under_domain_level_0_keeps_ca-related_RUVs
-        """
-        master = self.master
-        replica = self.replicas[0]
-        replica.run_command(['ipactl', 'stop'])
-        master.run_command(['ipa-replica-manage', 'del', '-f', '-p',
-                            master.config.dirman_password, replica.hostname])
-        result = master.run_command(['ipa-replica-manage', 'list-ruv',
-                                     '-p', master.config.dirman_password])
-        num_ruvs = result.stdout_text.count(replica.hostname)
-        assert(num_ruvs == 1), ("Expected to find 1 replica's RUV, found %s" %
-                                num_ruvs)
-        ruvid_re = re.compile(".*%s:389: (\d+).*" % replica.hostname)
-        replica_ruvs = ruvid_re.findall(result.stdout_text)
-        master.run_command(['ipa-replica-manage', 'clean-ruv', '-f',
-                            '-p', master.config.dirman_password,
-                            replica_ruvs[0]])
-        result2 = master.run_command(['ipa-replica-manage', 'list-ruv',
-                                      '-p', master.config.dirman_password])
-        assert(replica.hostname not in result2.stdout_text), (
-            "Replica's RUV was not properly removed")
-
-    def test_clean_dangling_ruv_multi_ca(self):
-        """
-        http://www.freeipa.org/page/V4/Manage_replication_topology_4_4/
-        Test_Plan#Test_case:_ipa-replica-manage_clean-dangling-ruv_in_a
-        _multi-CA_setup
-        """
-        master = self.master
-        replica = self.replicas[1]
-        replica.run_command(['ipa-server-install', '--uninstall', '-U'])
-        master.run_command(['ipa-replica-manage', 'del', '-f', '-p',
-                            master.config.dirman_password, replica.hostname])
-        result1 = master.run_command(['ipa-replica-manage', 'list-ruv', '-p',
-                                      master.config.dirman_password])
-        ruvid_re = re.compile(".*%s:389: (\d+).*" % replica.hostname)
-        assert(ruvid_re.search(result1.stdout_text)), (
-            "Replica's RUV should not be removed under domain level 0")
-        master.run_command(['ipa-replica-manage', 'clean-dangling-ruv', '-p',
-                            master.config.dirman_password], stdin_text="yes\n")
-        result2 = master.run_command(['ipa-replica-manage', 'list-ruv', '-p',
-                                      master.config.dirman_password])
-        assert(replica.hostname not in result2.stdout_text), (
-            "Replica's RUV was not removed by a clean-dangling-ruv command")
-
-    def test_replica_managed_del_domlevel1(self):
-        """
-        http://www.freeipa.org/page/V4/Manage_replication_topology_4_4/
-        Test_Plan#Test_case:_ipa-replica-manage_del_with_turned_off_replica
-        _under_domain_level_1_removes_ca-related_RUVs
-        """
-        master = self.master
-        replica = self.replicas[2]
-        master.run_command(['ipa', 'domainlevel-set', '1'])
-        replica.run_command(['ipactl', 'stop'])
-        master.run_command(['ipa-replica-manage', 'del', '-f', '-p',
-                            master.config.dirman_password, replica.hostname])
-        result = master.run_command(['ipa-replica-manage', 'list-ruv',
-                                     '-p', master.config.dirman_password])
-        assert(replica.hostname not in result.stdout_text), (
-            "Replica's RUV was not properly removed")
